#!/bin/bash
# Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
# Software License, Version 1.0. (See accompanying file
# LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
#
app_name=$(basename $0)
#
# surprisingly, prints the usage screen
function print_usage()
{
	cat $(dirname $0)/maureen-module/maureen.txt
	return 0
}
#
# gets the default toolset
function default_toolset()
{
	echo "doxyreg"
	return 0
}
#
# a temporary file that is going to be passed to the toolset as input
tempfile=""
# the list of MAREfiles that will be appended to the tempfile
marefiles="$(find . -maxdepth 1 -type f -name MAREfile 2> /dev/null)"
prev_input=false
#
function append_input()
{
	if [ "${prev_input}" == "true" ]
	then echo "Input option must be followed by an output option." && return 1
	fi
	echo -n "'${1}' " >> ${tempfile:=$(mktemp)}
	prev_input=true
	return 0
}
#
function append_output()
{
	if [ "${prev_input}" == "false" ]
	then echo "Output option must be preceded by an input option." && return 2
	fi
	echo "'${1}'" >> ${tempfile:=$(mktemp)}
	prev_input=false
	return 0
}
#
function add_marefile()
{
	if [ ! -r "${1}" ]
	then echo "Cannot read file '${1}'" && return 3
	fi
	if [ "${marefiles}" == "" ]
	then marefiles="${1}"
	elif [ "${marefiles}" == "./MAREfile" ]
	then marefiles="${1}"
	else marefiles="${marefiles} ${1}"
	fi
	return 0
}
#
function append_option()
{
	echo "@ '${1/=/|}'" >> ${tempfile:=$(mktemp)}
	return 0
}
#
function set_macro_prefix()
{
	if [ "${macro_prefix}" != "" ]
	then echo "Multiple macro prefixes provided!" && return 4
	fi
	macro_prefix=${1}
	append_option "macro_prefix|${macro_prefix}"
	return 0
}
#
long_option_pattern="\
toolset:,\
input:,\
output:,\
marefile:,\
include-path:,\
macro-prefix:,\
boost-macro-prefix,\
define:,\
debug-tempfile,\
help\
"
if [ -x "$(which getopt 2> /dev/null)" ]
then
	# parse options with getopt
	parsed_options=$(getopt \
		--name=${app_name} \
		--options=t:,i:,o:,m:,I:,p:,b,D:,h \
		--longoptions=${long_option_pattern} \
		-- "$@" \
	)
	#
	# check the result and exit on error
	[[ $? -eq 0 ]] || exit $?
	#
	# use the pre-parsed options
	eval set -- ${parsed_options}
fi
# now parse them here
while [ $# -gt 0 ]
do
	case ${1} in
		-D|--define) shift && append_option "define|${1}" || exit $?;;
		-D*) append_option "define|${1:2}" || exit $?;;
		-i|--input)  shift && append_input   ${1} || exit $?;;
		-i*) append_input ${1:2} || exit $?;;
		-o|--output) shift && append_output  ${1} || exit $?;;
		-o*) append_output ${1:2} || exit $?;;
		-m|--marefile) shift && add_marefile ${1} || exit $?;;
		-m*) add_marefile ${1:2} || exit $?;;
		-I|--include-path) shift && append_option "include|${1}" || exit $?;;
		-I*) append_option "include|${1:2}" || exit $?;;
		-p|--macro-prefix) shift && set_macro_prefix ${1} || exit $?;;
		-p*) set_macro_prefix ${1:2} || exit $?;;
		-b|--boost-macro-prefix) set_macro_prefix BOOST_ || exit $?;;
		-t|--toolset) shift && toolset=${1};;
		-t*) toolset=${1:2};;
		-h|--help) print_usage && exit $?;;
		--) shift && break;;
		--debug-tempfile) debug_tempfile_only=true;;
		 *) echo "Invalid option '${1}'" && exit 100;;
	esac
	shift
done
#
# check if the last input has a matching output
if [ "${prev_input}" == "true" ]
then echo "Missing output option for input option." && exit 1
fi
#
# if we have a MAREfile append it to the tempfile
for marefile in ${marefiles}
do
	if [[ -r "${marefile}" && -r "${tempfile:=$(mktemp)}" ]]
	then
		root_prefix=$(dirname ${marefile})
		apply_prefix_sed="
			/^#.*/d
			s#^'\([^/|%]\+\)#'${root_prefix}/\1#
			s#^'|#'${root_prefix}|#
			s#'\s\+'\([^|]\+\)#' '${root_prefix}/\1#
			s#'\s\+'|#' '${root_prefix}|#
		"
		if [ "${root_prefix}" != "." ]
		then sed "${apply_prefix_sed}" < ${marefile} >> ${tempfile}
		else grep -e '^#' -v ${marefile} >> ${tempfile}
		fi
	fi
done
# remove the temporary file if any at exit
trap "rm -f '${tempfile}'" EXIT
#
# check if we have some (nonempty) input
if [[ ! -r "${tempfile}" || $(grep -c -v -e '^\s*$' -e'^@' < ${tempfile}) -eq 0 ]]
then echo "No input." && exit 4
fi
#
if [ "${debug_tempfile_only}" == "true" ]
then cat ${tempfile}; exit $?
fi
# use the default toolset if none was provided
toolset=${toolset:-$(default_toolset)}
# find the toolset executable
for exe in $(find \
	$(dirname $0)/maureen-module \
	-maxdepth 1 \
	-type f \
	-executable \
	-perm -u+x \
	-regex ".*/${toolset}\.\w*" \
)
do
	case "${toolset_executable}" in
		*.sh) break;;
		*) toolset_executable=${exe};;
	esac
done
# if found
if [ -x "${toolset_executable}" ]
# call it with the rest of the arguments
then (
	grep ${tempfile} -e'^@'
	echo @ -
	grep ${tempfile} -e'^[^@]'
) | "${toolset_executable}" $*
# otherwise complain
else echo "Could not find any executable for toolset '${toolset}'." && exit 5
fi
