/**
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef MIRROR_EXAMPLE_FACTORIES_TETRAHEDRON_1011291729_HPP
#define MIRROR_EXAMPLE_FACTORIES_TETRAHEDRON_1011291729_HPP

#include <cmath>

namespace test {

struct vector
{
	double x,y,z;

	vector(double _x, double _y, double _z)
	 : x(_x)
	 , y(_y)
	 , z(_z)
	{ }

	vector(double _w)
	 : x(_w)
	 , y(_w)
	 , z(_w)
	{ }

	vector(void)
	 : x(0.0)
	 , y(0.0)
	 , z(0.0)
	{ }

	// addition
	friend vector operator + (const vector& a, const vector& b)
	{
		return vector(a.x + b.x, a.y + b.y, a.z + b.z);
	}

	// subtraction
	friend vector operator - (const vector& a, const vector& b)
	{
		return vector(a.x - b.x, a.y - b.y, a.z - b.z);
	}

	// cross product
	friend vector operator % (const vector& a, const vector& b)
	{
		return vector(
			a.y * b.z - a.z * b.y,
			a.z * b.x - a.x * b.z,
			a.x * b.y - a.y * b.x
		);
	}

	// dot product
	friend double operator * (const vector& a, const vector& b)
	{
		return a.x*b.x + a.y*b.y + a.z*b.z;
	}

	double length(void) const
	{
		return std::sqrt(x*x + y*y + z*z);
	}
};

struct triangle
{
	vector a, b, c;

	triangle(const vector& _a, const vector& _b, const vector& _c)
	 : a(_a)
	 , b(_b)
	 , c(_c)
	{ }

	triangle(void){ }


	double area(void) const
	{
		return ((b-a)%(c-a)).length()/2.0;
	}
};

struct tetrahedron
{
	triangle base;
	vector apex;

	tetrahedron(const triangle& _base, const vector& _apex)
	 : base(_base)
	 , apex(_apex)
	{ }

	tetrahedron(
		const vector& a,
		const vector& b,
		const vector& c,
		const vector& d
	): base(a, b, c)
	 , apex(d)
	{ }

	const vector& a(void) const {return base.a;}
	const vector& b(void) const {return base.b;}
	const vector& c(void) const {return base.c;}
	const vector& d(void) const {return apex;}

	void reset_apex(const vector& _apex)
	{
		apex = _apex;
	}

	void reset_apex(double x, double y, double z)
	{
		apex = vector(x, y, z);
	}

	double volume(void) const
	{
		return std::fabs(((a()-d())*((b()-d())%(c()-d()))))/6.0;
	}
};

} // namespace test

#endif

